const {
  encrypt,
  generateHash,
  checkHash,
} = require("../../config/auth/cryptography");
const { genereteToken } = require("../../config/auth/jwt");
const { decryptDonor } = require("../../util/cryptDonor");
const db = require("../database/db");
const nodemailer = require("nodemailer");
const crypto = require("crypto");
const path = require("path");
const axios = require('axios');
const fs = require("fs");
const xml2js = require('xml2js');
const bcrypt = require('bcrypt');
const soap = require('soap');
const { sendMessage } = require("../../util/gzappy")
const { validarCNPJ } = require("../../util/validarCNPJ");
const { validarCPF } = require("../../util/validarCPF");
const { validarNumeroCelularLib, validarNumeroCelularRegex } = require("../../util/validacaoCelularLib");
const { saveLastId, getLastId } = require("../../util/geradorId");
const { removeUltimoDigitoFloat } = require("../../util/removeUltimoDigito");
require("dotenv").config({ path: __dirname + "/./../../../.env" });


class usersRepository {
  constructor() {
    this.db = db;
    this.wsdlUrl = process.env.SOAP_WSDL_URL;
    this.soapAction = process.env.SOAP_ACTION;
  }

  /**
   * Registra os dados de um usuario
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  register(data) {
    return new Promise(async (resolve, reject) => {
  
      const resultado = 
        data.documento.length === 11 ? validarCPF(data.documento) :
        data.documento.length === 14 ? validarCNPJ(data.documento) : null;
  
      const celularValido = validarNumeroCelularLib(data.celular);
  
      if (!celularValido) return resolve({ error: "Celular Inválido!", code: 203 });
      if (!resultado) return resolve({ error: "CNPJ ou CPF inexistente!", code: 203 });
  
      const hashedPassword = await bcrypt.hash(data.senha, 10);
      const dateAtual = new Date();
  
      // Define valores padrão para notificações
      const notificacaoWhats = typeof data.notificacao_whats === "boolean" ? data.notificacao_whats : true;
      const notificacaoEmail = typeof data.notificacao_email === "boolean" ? data.notificacao_email : false;
  
      this.db.query(
        `INSERT INTO user (razao_social, documento, celular, email, role, senha, endereco, anotacoes, data_criado, notificacao_email, notificacao_whats) VALUES(?,?,?,?,?,?,?,?,?,?,?)`,
        [
          data.razao_social,
          data.documento,
          data.celular,
          data.email,
          "USER",
          hashedPassword,
          data.endereco,
          data.anotacoes,
          dateAtual,
          notificacaoEmail,
          notificacaoWhats
        ],
        async (error, response) => {
          if (error) {
            console.error(error);
            return resolve({ error: "Erro ao cadastrar usuario", code: 203 });
          }
          return resolve({ success: "Usuario cadastrado com sucesso!", code: 200 });
        }
      );
    });
  }
  


 
  /**
   * Realiza o login de um usuario
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  login(data, ip) {
    return new Promise(async (resolve, reject) => {

      let celular_vendedor;

      this.db.query(
        "SELECT * FROM user WHERE documento=?",
        [data.documento, data.senha],
        async (error, response) => {
          if (error) {
            console.error("Database query error:", error);
            return reject(new Error("Database query error"));
          }

          if (response.length === 0) return resolve({ error: "Usuario não encontrado", code: 204 });
          

          
          if (response[0].id_vendedor) {
          
            celular_vendedor = await new Promise((resolve, reject) => {
            this.db.query(
              `SELECT celular AS celular_vendedor FROM vendedores WHERE id = ?`,
              [response[0].id_vendedor],
              (error, results) => {
                if (error) {
                  console.error("Error fetching existing order:", error);
                  return reject(error);
                }
                resolve(results[0].celular_vendedor);
              }
            );
          });
        } else {
            celular_vendedor = null
        }
          const userDB = response[0];  

          const hashedPassword = await bcrypt.hash(data.senha, 10);

          if (!userDB.senha) {
          this.db.query(
            "UPDATE vendedores SET senha=? WHERE email=?",
            [hashedPassword, data.email],
            (updateError) => {
              if (updateError) {
                console.error("Error updating password: ", updateError);
                return reject(new Error("Error updating password"));
              }
            }
          );
        }
        else {
          const isMatch = await bcrypt.compare(data.senha, userDB.senha);
          if (!isMatch) return resolve({ error: "Senha incorreta", code: 401 });

        }


          const auth = crypto.randomBytes(128).toString("HEX");
          userDB.auth = auth;

          // Atualiza o banco de dados com o novo auth
          this.db.query(
            "UPDATE user SET auth=? WHERE documento=?",
            [auth, data.documento],
            async (updateError) => {
              if (updateError) {
                console.error("Database update error:", updateError);
                return reject(new Error("Database update error"));
              }

              this.db.query(
                `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
                [userDB.id, new Date(), "Fez login"],
                (error, results) => {
                  if (error) {
                    console.error("Error inserting logs: ", error);
                    return reject(error);
                  }
                  resolve();
                }
              );

              // Gera o token
              try {
                const token = await genereteToken(userDB);

                // Retorna todos os detalhes do usuário junto com o token
                return resolve({
                  token: token,
                  user: {
                    id: userDB.id,
                    razao_social: userDB.razao_social,
                    documento: userDB.documento,
                    email: userDB.email,
                    role: userDB.role,
                    celular: userDB.celular,
                    status: userDB.status,
                    endereco: userDB.endereco,
                    qtde_minima_suino: userDB.qtde_minima_suino,
                    qtde_minima_bovino: userDB.qtde_minima_bovino,
                    tipo: userDB.tipo,
                    anotacoes: userDB.anotacoes,
                    id_vendedor: userDB.id_vendedor,
                    celular_vendedor: celular_vendedor,
                    aroma: userDB.aroma,
                    auth: auth,
                  },
                });
              } catch (tokenError) {
                console.error("Token generation error:", tokenError);
                return reject(new Error("Token generation error"));
              }
            }
          );
        }
      );
    });
  }

  /**
   * Realiza o logout do usuário
   * @param {*} data email do usuário
   * @returns mensagem referente ao resultado obtido
   */
  logout(auth) {
    return new Promise(async (resolve, reject) => {
      
      this.db.query(
        "UPDATE user set auth=null WHERE auth=?",
        [auth],
        async (error, response) => { 
          if (error) return reject(new Error(error));
          return resolve({ success: "Logout realizado com sucesso!" });
        }
      );
    });
  }

  /**
   * Envia um email para recuperação de senha
   * @param {*} data dados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  recover(data) {
    return new Promise(async(resolve, reject) => {
      // Verificar se o email existe no banco de dados
      this.db.query("SELECT email, celular FROM user WHERE documento = ?", [data.documento], 
        async (err, result) => {
        if (err) return reject("Erro no banco de dados");

        if (result.length === 0) {
          return resolve({ error: "Email não encontrado!", code: 204 });
        }

        let newPassword = Math.floor(100000 + Math.random() * 900000);
        newPassword += ""

   
        const user = result[0];

        const novaMensagem = `*Senha Redefinida*

Sua senha temporária é: *${newPassword}*

Caso não tenha solicitado, entre em contato com *(11) 97259-7464*`
        

        await sendMessage(novaMensagem, "+55"+user.celular)
       
        const hashedPassword = await bcrypt.hash(newPassword, 10);

        // Atualizar a senha no banco de dados
        db.query("UPDATE user SET senha = ? WHERE documento = ?", [hashedPassword, data.email], (err) => {
          if (err) {
            console.log(err);

            return reject("Erro ao atualizar a senha no banco de dados");
          }

          // Configuração do transporte de email
          let transporter = nodemailer.createTransport({
            host: 'brasapedidos.com.br',
            port: 465,
            secure: true,     
            auth: {
              user: process.env.EMAIL_SENDER,
              pass: process.env.EMAIL_PASSWORD,      
            },
          });

          let mailOptions = {
            from: 'suporte@brasapedidos.com.br',
            to: data.email,
            subject: 'Recuperação de senha',
            // text: `${newPassword}`,
            html: `
            <!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    <title>Password Recovery</title>
    <style>
        body {
            font-family: Arial, sans-serif;
            background-color: #f4f4f4;
            margin: 0;
            padding: 0;
        }
        .container {
            max-width: 500px; /* Define a largura máxima */
            width: 100%;
            margin: 40px auto;
            background-image: linear-gradient(180deg, #B91C1C, #DC2626);
            color: white;
            padding: 20px;
            border-radius: 8px;
            box-shadow: 0 4px 8px rgba(0, 0, 0, 0.1);
        }
        h1 {
            font-size: 24px;
            color: #f3f4f6;
            text-align: center;
            margin-bottom: 20px;
        }
        p {
            font-size: 16px;
            color: #f3f4f6;
            line-height: 1.5;
            margin: 0 0 20px;
        }
        .new-password {
            display: block;
            font-size: 20px;
            font-weight: 600;
            color: #ffffff;
            border: 2px solid rgba(255, 255, 255, 0.7);
            background-color: rgba(255, 255, 255, 0.1);
            padding: 15px;
            border-radius: 4px;
            text-align: center;
            margin: 20px 0;
        }
        .footer {
            text-align: center;
            font-size: 14px;
            color: #f3f4f6;
            margin-top: 20px;
        }
    </style>
</head>
<body>
    <div class="container">
        <h1>Senha Alterada</h1>
        <p>Sua senha foi alterada com sucesso!</p>
        <p>Sua nova senha é:</p>
        
        <div class="new-password">
            ${newPassword}
        </div>
        
        <p>Se você não solicitou essa mudança, ignore este e-mail.</p>
        
        <div class="footer">
            &copy; 2024 BFS Alimentos
        </div>
    </div>
</body>
</html>
            `
          };

          transporter.sendMail(mailOptions, (error, info) => {
            if (error) {
              return console.log(error);
            }
            console.log('Message sent: %s', info.messageId);
          });

          return resolve({ success: "Nova senha enviada para o email!" });

        });
      });
    });
  }

  ultra(data) {
    return new Promise(async (resolve, reject) => {

        this.db.query(
            `INSERT INTO produtos_ultra (
                codProduto, 
                codProdutoClass, 
                dscProduto, 
                marca, 
                undMedida, 
                preco, 
                qtdEstoque, 
                codGrupoNivel1, 
                codGrupoNivel2, 
                codGrupoNivel3, 
                grupoFiscal, 
                suspenderPedidos, 
                idnForaLinha, 
                dataBaixa, 
                refFabrica, 
                idnRevenda, 
                descricaoClasse, 
                linha, 
                coluna, 
                idnGrade, 
                idnFoto, 
                codbarras, 
                preco_minimo, 
                custo, 
                obs, 
                markupMinimo, 
                markupPadrao, 
                markupVenda, 
                sequencia, 
                pesoProduto, 
                pesoEmbalagem, 
                ecommerceAtivo, 
                secao, 
                largura, 
                altura, 
                comprimento, 
                produto_aplicacao, 
                produto_clas_aplicacao
            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`,
            [
                data.codProduto,
                data.codProdutoClass,
                data.dscProduto,
                data.marca,
                data.undMedida,
                data.preco,
                data.qtdEstoque,
                data.codGrupoNivel1,
                data.codGrupoNivel2,
                data.codGrupoNivel3,
                data.grupoFiscal,
                data.suspenderPedidos,
                data.idnForaLinha,
                data.dataBaixa,
                data.refFabrica,
                data.idnRevenda,
                data.descricaoClasse,
                data.linha,
                data.coluna,
                data.idnGrade,
                data.idnFoto,
                data.codbarras,
                data.preco_minimo,
                data.custo,
                data.obs,
                data.markupMinimo,
                data.markupPadrao,
                data.markupVenda,
                data.sequencia,
                data.pesoProduto,
                data.pesoEmbalagem,
                data.ecommerceAtivo,
                data.secao,
                data.largura,
                data.altura,
                data.comprimento,
                data.produto_aplicacao,
                data.produto_clas_aplicacao
            ],
            async (error, response) => {
                if (error) {
                    console.error(error);
                    return resolve({ error: "Erro ao cadastrar produto", code: 203 });
                }
                return resolve({ success: "Produto cadastrado com sucesso!", code: 200 });
            }
        );
    });
}

    /**
* Lista os dados de usuarios aprovados
* @returns lista com todos os dados de um usuario
*/
listUserIds() {
  return new Promise(async (resolve, reject) => {


    try {
      this.db.query(
        `SELECT id FROM user WHERE user.role != 'ADMIN'`,
        [],

        (error, id_usuarios) => {
          if (error) return reject(new Error(error));
          return resolve(id_usuarios);
        }
      );
    } catch (error) {
      return reject(new Error(error));
    }
  });
}


  /**
   * Lista os dados de um usuario
   * @returns lista com todos os dados de um usuario
   */
  list(id) {
    return new Promise(async (resolve, reject) => {
      var query = id ? `WHERE id = ?` : "";


      try {
        this.db.query(
          `SELECT * FROM user ${query}`,
          [id ? id : ""],

          (error, usuarios) => {
            if (error) return reject(new Error(error));

            for (const usuario of usuarios) {

              if (usuario.image !== "") {
                const url = "https://aliancapedidos.s3.us-east-2.amazonaws.com/"+usuario.image
                usuario.image = url
              }
              else usuario.image = ""
            }
            return resolve(usuarios);
          }
        );
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }

  listVendedorPorUserId(id_user) {
    return new Promise(async (resolve, reject) => {


      try {

        const id_vendedor = await new Promise(async (resolve, reject) => {
        this.db.query(
          `SELECT id_vendedor FROM user WHERE id = ?`,[id_user],
          (error, usuario) => {
            if (error) return reject(new Error(error));

            resolve(usuario[0].id_vendedor);
          });
      });
       
      if (id_vendedor == null) return resolve({})
      
      const vendedor = await new Promise(async (resolve, reject) => {
        this.db.query(
          `SELECT id, nome, celular, email FROM vendedores WHERE id = ?`,[id_vendedor],
          (error, usuario) => {
            if (error) return reject(new Error(error));

            resolve(usuario[0]);
          });
      });

      return resolve(vendedor)
      } catch (error) {
        return reject(new Error(error));
      }
    });
  }


  async listUserInfo(id_user) {
    return new Promise(async (resolve, reject) => {
      try {
        const userInfo = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT 
              (SELECT COUNT(*) FROM pedidos WHERE concluido = true AND id_user = ?) AS pedidos_concluido,
              (SELECT SUM(quantidade) FROM pedidos WHERE concluido = true AND id_user = ?) AS produtos_quantidade,
              (SELECT DATE_FORMAT(MAX(data), "%d/%m/%Y") FROM pedidos WHERE concluido = true AND id_user = ?) AS ultimo_pedido`,
            [id_user, id_user, id_user],
            (error, result) => {
              if (error) return reject(error);
              resolve(result[0]);
            }
          );
        });
  
        const topProdutos = await new Promise((resolve, reject) => {
          this.db.query(
            `SELECT pr.id AS id_produto, pr.nome AS produto, SUM(p.quantidade) AS quantidade 
             FROM pedidos p
             JOIN produtos pr ON p.id_produto = pr.id
             WHERE p.concluido = true AND p.id_user = ?
             GROUP BY pr.id, pr.nome
             ORDER BY quantidade DESC
             LIMIT 5`,
            [id_user],
            (error, result) => {
              if (error) return reject(error);
              resolve(result);
            }
          );
        });
  
        resolve({
          pedidos_concluido: userInfo.pedidos_concluido || 0,
          produtos_quantidade: userInfo.produtos_quantidade || 0,
          ultimo_pedido: userInfo.ultimo_pedido || "N/A",
          topProdutos: topProdutos
        });
      } catch (error) {
        reject(error);
      }
    });
  }
  

  /**
   * Deleta o usuario cadastrado
   * @param {*} id ID do usuario a ser excluído
   * @returns mensagem referente ao resultado obtido
   */
  delete(id) {
    return new Promise((resolve, reject) => {
      this.db.query(
        "DELETE FROM user WHERE id = ?",
        [id],
        (error, response) => {
          if (error) {
            return resolve({
              error: "Este usuarios possui vinculos!",
              code: 203,
            });
          } else {
            return resolve(true);
          }
        }
      );
    });
  }

  /**
   * Atualiza os dados de um usuario
   * @param {*} data dados atualizados do usuario
   * @returns mensagem referente ao resultado obtido
   */
  update(data) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        "UPDATE user SET razao_social=?, celular = ?, email=?, endereco = ?, anotacoes = ? WHERE id=?",
        [data.razao_social, data.celular, data.email, data.endereco, data.anotacoes, data.id],
        async (error, response) => {
          if (error) return reject(new Error(error));

          this.db.query(
            `INSERT INTO logs (id_user, data, acao) VALUES (?,?,?)`,
            [data.id, new Date(), "Atualizou o proprio perfil"],
            (error, results) => {
              if (error) {
                console.error("Error inserting logs: ", error);
                return reject(error);
              }
              resolve();
            }
          );
          
          return resolve({ success: "Usuario atualizado com sucesso!", code: 200 });
        }
      );
    });
  }

  /**
   * Verifica o código enviado por email e valida
   * @param {*} code código recebido no email
   * @returns mensagem referente ao resultado obtido
   */
  verify(code) {
    return new Promise((resolve, reject) => {
      this.db.query(
        "SELECT * FROM usuarios WHERE codigo=?",
        [code],
        async (error, response) => {
          if (error) return reject(new Error(error));
          try {
            if (response[0]) {
              this.db.query(
                "UPDATE usuarios SET codigo=true WHERE codigo=?",
                [code],
                (error, response) => {
                  if (error) return reject(new Error(error));
                  return resolve({ success: `Código verificado!` });
                }
              );
            } else
              return resolve({
                error: "Não foi possível verificar!",
                code: 203,
              });
          } catch (error) {
            return reject(new Error(error));
          }
        }
      );
    });
  }

  /**
   * Altera a senha de um usuário
   * @param {*} data dados do usuário
   * @param {*} admin
   * @returns
   */
  changePassword( data, id_user) {
    return new Promise(async (resolve, reject) => {
      const hashedPassword = await bcrypt.hash(data.senha, 10);
      this.db.query(
        "UPDATE user SET senha=? WHERE id=?",
        [hashedPassword, id_user],
        async (error, response) => {
          if (error) return reject(new Error(error));
          return resolve({ success: `Senha alterada!`, code: 200 });
        }
      );
    });
  }


  /**
   * Lista quantidade minima de produto que um user pode comprar
   * @param {*} id id do usuario
   * @returns quatidade minima de produto de um usuario
   */
  listQtdeMinima(id) {
    return new Promise(async (resolve, reject) => {
      this.db.query(
        `SELECT user.id, user.qtde_minima_suino, user.qtde_minima_bovino FROM user WHERE id = ?`,
        [id],
        async (error, user) => {
          if (error) return reject(new Error(error));
          return resolve(user);
        }
      );
    });
  }

  /**
   * Criptografa um valor
   * @param {*} data valor recebido
   * @returns o valor criptografado
   */
  encrypt(data) {
    return new Promise(async (resolve, reject) => {
      try {
        let value;
        if (data.hash) {
          value = await generateHash(data.value);
        } else {
          value = await encrypt(data.value);
        }
        return resolve({ value: value });
      } catch (error) {
        return resolve({ error: `Não foi possível criptografar!`, code: 203 });
      }
    });
  }


  async createClient() {
const soapAction = 'http://www.ultrasistemas.com.br/';
const soapEnvelope = `
  <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
    <soap12:Body>
      <TesteConexao xmlns="http://www.ultrasistemas.com.br/">
        <codigo>10156</codigo>
        <senha>1234</senha>
      </TesteConexao>
    </soap12:Body>
  </soap12:Envelope>`;


  try {
    const response = await axios.post(this.wsdlUrl, soapEnvelope, {
      headers: {
        'Content-Type': 'application/soap+xml; charset=utf-8',
        'Content-Length': soapEnvelope.length,
        'SOAPAction': soapAction, // SOAP Action obrigatório
      },
    });

    console.log('SOAP Response: ', response);
    return response
  } catch (error) {
    console.error('Erro ao testar SOAP:', error.message);
    if (error.response) {
      console.error('Resposta do servidor:', error.response.data);
    }
  }
}


async saveAllOrders(id_user) {

  const pedido = await new Promise((resolve, reject) => {
    this.db.query(
        `SELECT pedidos.id_user, pedidos.id_produto, pedidos.quantidade FROM pedidos WHERE concluido = false AND id_user = ?`, [id_user],
        (error, pedido) => {
            if (error) {
                console.error("Error fetching orders: ", error);
                return reject(error);
            }
            resolve(pedido);
        }
    );
});

const descontos = await Promise.all(
    pedido.map(p =>
        new Promise((resolve, reject) => {
            this.db.query(
                `SELECT * FROM descontos WHERE id_produto = ? AND id_user = ?`, [p.id_produto, id_user],
                (error, desconto) => {
                    if (error) {
                        console.error(`Error fetching discount for product ${p.id_produto}: `, error);
                        return reject(error);

                        
                    }
                    resolve(desconto[0] || { percentual: 0 }); // Retorna 0% desconto se não houver registro.
                }
            );
        })
    )
);

const produtos = await Promise.all(
    pedido.map(p =>
        new Promise((resolve, reject) => {
            this.db.query(
                `SELECT * FROM produtos_ultra WHERE codProduto = ?`, [p.id_produto],
                (error, result) => {
                    if (error) {
                        console.error(`Error fetching product ${p.id_produto}: `, error);
                        return reject(error);
                    }
                    resolve(result[0]);
                }
            );
        })
    )
);


let ultimoId = await getLastId()+1

const itens = produtos.map((produto, index) => {
  const precoOriginal = produto.preco;
  const qtdPedido = pedido[index].quantidade;
  const descontoPercentual = descontos[index].desconto || 0; 

  const valorDescontoUnitario = (precoOriginal * descontoPercentual) / 100;

  // Preço líquido por unidade após o desconto
  const precoLiquidoUnitario = precoOriginal - valorDescontoUnitario;

  // Cálculo do valor total (para a quantidade do pedido)
  const valorTotalOriginal = precoOriginal * qtdPedido;
  const valorDesconto = valorDescontoUnitario * qtdPedido;
  const valorLiquido = precoLiquidoUnitario * qtdPedido;

  console.log("VALOR DESCONTO: ", valorDesconto);
  console.log("VALOR LIQUIDO: ", valorLiquido);

  console.log("VALOR DESCONTO UNI ", valorDescontoUnitario);
  
  
  return {
      codGrupoNivel1: produto.codGrupoNivel1,
      codGrupoNivel2: produto.codGrupoNivel2,
      codGrupoNivel3: produto.codGrupoNivel3,
      codPedido: ultimoId,
      codProduto: produto.codProduto,
      codProdutoClas: produto.codProdutoClass,
      codTributacaoUF: 0,
      codTributacaoUFDest: 0,
      codUnidadeMedida: 0,
      descUnidadeMedida: "",
      grupoFiscal: produto.grupoFiscal,
      item: index + 1,
      margemST: 0.0,
      perIPI: 0.0,
      percBaseICMSUF: 0.0,
      percBaseICMSUFDest: 0.0,
      percICMSUF: 0.0,
      percICMSUFDest: 0.0,
      pesoEmbalagem: produto.pesoEmbalagem,
      pesoRepesagem: "0,0000",
      preco: precoLiquidoUnitario,
      qtdAtual: 0.0,
      qtdPedido: qtdPedido,
      qtdUnMed: produto.pesoProduto,
      quantidade: qtdPedido,
      unidadeMedida: produto.undMedida,
      valorBaseICMSUF: 0.0,
      valorBaseICMSUFDest: 0.0,
      valorBaseIPI: 0.0,
      valorBaseST: 0.0,
      valorBruto: valorLiquido.toFixed(2),
      valorDesconto: valorDescontoUnitario,
      valorICMSUF: 0.0,
      valorICMSUFDest: 0.0,
      valorIPI: 0.0,
      valorInformado: precoLiquidoUnitario,
      valorLiquido: valorLiquido.toFixed(2),
      valorRepesagem: "0.00",
      valorST: 0.0,
      valorTotal: valorLiquido.toFixed(2),
      vlrAtual: 0.0,
      vlrTotalAtual: 0.0
  };
});

// Calcula o valor líquido total para o envelope SOAP.
const valorLiquidoTotal = itens.reduce((acc, item) => acc + parseFloat(item.valorLiquido), 0);

const dataFormatada = new Date().toISOString().split('T')[0];
const dataAtual = new Date();

dataAtual.setDate(dataAtual.getDate() + 2);

const dataEntrega = dataAtual.toISOString().split('T')[0];

const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
xmlns:xsd="http://www.w3.org/2001/XMLSchema"
xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
<soap12:Body>
<SaveAllOrders xmlns="http://www.ultrasistemas.com.br/">
<strPedidos>
[{
"codCondRec": 4,
"codFilial": 1,
"codForma": 2,
"codOperacao": 100,
"codTabelaPreco": 1,
"codVendedor": 10156,
"cpfCnpj": "372.236.674-78",
"data": "${dataFormatada}",
"dataEntrega": "${dataEntrega}",
"emailCliente": "",
"id":  ${ultimoId},
"itens": ${JSON.stringify(itens)},
"md5": "${crypto.randomBytes(9).toString('hex').slice(0, 12)}",
"newCodigoFV": 0,
"observacao": "PEDIDO TESTE NAO FATURAR",
"parceiro": 10204,
"codigo": ${ultimoId},
"pesoBruto": 0.0,
"pesoLiquido": 0.0,
"qtdeVolumes": 0,
"valorBaseST": 0.0,
"valorDesconto": "${itens.reduce((acc, item) => acc + parseFloat(item.valorDesconto), 0).toFixed(2)}",
"valorIPI": 0.0,
"valorBruto": "0.00",
"valorST": 0.0,
"valorLiquido": "0.00"
}]
</strPedidos>
</SaveAllOrders>
</soap12:Body>
</soap12:Envelope>`;


await saveLastId(ultimoId);


    try {
      const response = await axios.post(this.wsdlUrl, soapEnvelope, {
        headers: {
          'Content-Type': 'application/soap+xml; charset=utf-8',
          'SOAPAction': this.soapAction, 
        },
      });
  
      console.log('SOAP Response: ', response);
      return response
    } catch (error) {
      if (error.response) {
        // O servidor respondeu com um status HTTP fora do intervalo 2xx
        console.error("Status:", error.response.status);
        // console.error("Headers:", error.response.headers);
        console.error("Dados da resposta:", error.response.data);
      } else if (error.request) {
        // A requisição foi feita, mas nenhuma resposta foi recebida
        // console.error("Nenhuma resposta do servidor. Detalhes da requisição:", error.request);
      } else {
        // Erro na configuração da requisição
        // console.error("Erro ao configurar a requisição:", error.message);
      }
    }
  }
  

  async getParceiro() {
    const soapAction = 'http://www.ultrasistemas.com.br/';
    const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
<soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
  <soap12:Body>
    <GetParceiros xmlns="http://www.ultrasistemas.com.br/">
      <dataHora>17-12-2024 14:00:00</dataHora>
      <codVendedor>10156</codVendedor>
    </GetParceiros>
  </soap12:Body>
</soap12:Envelope>`;
    
    
      try {
        const response = await axios.post(this.wsdlUrl, soapEnvelope, {
          headers: {
            'Content-Type': 'application/soap+xml; charset=utf-8',
            'Content-Length': soapEnvelope.length,
            'SOAPAction': soapAction, 
          },
        });
    
        console.log('SOAP Response: ', response);
        return response.data
      } catch (error) {
        console.error('Erro ao testar SOAP:', error.message);
        if (error.response) {
          console.error('Resposta do servidor:', error.response.data);
        }
      }
    }

    async getProdutos() {
    const soapAction = 'http://www.ultrasistemas.com.br/getProdutos';

    const dateAtual = new Date();
    const horaFormatada = dateAtual.toLocaleTimeString('pt-BR', { hour: '2-digit', minute: '2-digit' });
    const dataFormatada = dateAtual.toISOString().split('T')[0];

      const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
      <soap12:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap12="http://www.w3.org/2003/05/soap-envelope">
        <soap12:Body>
          <GetProdutos xmlns="http://www.ultrasistemas.com.br/">
            <dataHora>${dataFormatada} ${horaFormatada}</dataHora>
            <codVendedor>10156</codVendedor>
            <filial>1</filial>
          </GetProdutos>
        </soap12:Body>
      </soap12:Envelope>`
      
      try {
        const response = await axios.post(this.wsdlUrl, soapEnvelope, {
          headers: {
            'Content-Type': 'application/soap+xml; charset=utf-8',
            'Content-Length': soapEnvelope.length,
            'SOAPAction': soapAction, // SOAP Action obrigatório
          },
        });

        const regex = /\{([^{}]*)\}/g;
        let resultados = [];
        let match;
    
        
        while ((match = regex.exec(response.data)) !== null) {
            resultados.push(match[1].trim()); // Adicionar o conteúdo das chaves ao array
        }


        let returnJSON = resultados.map(item => {
          
          const jsonString = `{${item}}`;
          
          const correctedString = jsonString.replace(/"(\d+),(\d+)"/g, (_, integer, decimal) => `"${integer}.${decimal}"`);
          
          return JSON.parse(correctedString);
      });

      for (let i = 0; i < returnJSON.length; i++) {
        console.log("ENTROU!");
    
        const produto = returnJSON[i];
    
        await new Promise((resolve, reject) => {
            // Primeiro verifica se o produto já existe
            db.query(
                `SELECT 1 FROM produtos_ultra WHERE codProduto = ?`,
                [produto.codProduto],
                (error, results) => {
                    if (error) {
                        console.error("Error checking product existence:", error);
                        return reject(error);
                    }
    
                    if (results.length > 0) {
                        db.query(
                            `UPDATE produtos_ultra
                            SET 
                                codProdutoClass = ?, dscProduto = ?, marca = ?, undMedida = ?, preco = ?,  
                                qtdEstoque = ?, codGrupoNivel1 = ?, codGrupoNivel2 = ?, codGrupoNivel3 = ?, 
                                grupoFiscal = ?, suspenderPedidos = ?, idnForaLinha = ?, dataBaixa = ?, refFabrica = ?, 
                                idnRevenda = ?, descricaoClasse = ?, linha = ?, coluna = ?, idnGrade = ?, idnFoto = ?, 
                                codbarras = ?, preco_minimo = ?, custo = ?, obs = ?, markupMinimo = ?, 
                                markupPadrao = ?, markupVenda = ?, sequencia = ?, pesoProduto = ?, 
                                pesoEmbalagem = ?, ecommerceAtivo = ?, secao = ?, largura = ?, altura = ?, 
                                comprimento = ?, produto_aplicacao = ?, produto_clas_aplicacao = ?
                            WHERE codProduto = ?`,
                            [
                                produto.codProdutoClass, produto.dscProduto, produto.marca, produto.undMedida, produto.preco,
                                produto.qtdEstoque, produto.codGrupoNivel1, produto.codGrupoNivel2, produto.codGrupoNivel3,
                                produto.grupoFiscal, produto.suspenderPedidos, produto.idnForaLinha, produto.dataBaixa, produto.refFabrica,
                                produto.idnRevenda, produto.descricaoClasse, produto.linha, produto.coluna, produto.idnGrade, produto.idnFoto,
                                produto.codbarras, produto.preco_minimo, produto.custo, produto.obs, produto.markupMinimo,
                                produto.markupPadrao, produto.markupVenda, produto.sequencia, produto.pesoProduto,
                                produto.pesoEmbalagem, produto.ecommerceAtivo, produto.secao, produto.largura, produto.altura,
                                produto.comprimento, produto.produto_aplicacao, produto.produto_clas_aplicacao,
                                produto.codProduto // Chave única para WHERE
                            ],
                            (error) => {
                                if (error) {
                                    console.error("Error updating product:", error);
                                    return reject(error);
                                }
                                console.log(`Produto atualizado: ${produto.codProduto}`);
                                resolve();
                            }
                        );
                    } else {
                        db.query(
                            `INSERT INTO produtos_ultra (
                                codProduto, codProdutoClass, dscProduto, 
                                marca, undMedida, preco,  
                                qtdEstoque, codGrupoNivel1, codGrupoNivel2, 
                                codGrupoNivel3, grupoFiscal, suspenderPedidos, 
                                idnForaLinha, dataBaixa, refFabrica, 
                                idnRevenda, descricaoClasse, linha, 
                                coluna, idnGrade, idnFoto, 
                                codbarras, preco_minimo, custo, 
                                obs, markupMinimo, markupPadrao, 
                                markupVenda, sequencia, pesoProduto, 
                                pesoEmbalagem, ecommerceAtivo, secao, 
                                largura, altura, comprimento, 
                                produto_aplicacao, produto_clas_aplicacao
                            ) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)`,
                            [
                                produto.codProduto, produto.codProdutoClass, produto.dscProduto,
                                produto.marca, produto.undMedida, produto.preco,
                                produto.qtdEstoque, produto.codGrupoNivel1, produto.codGrupoNivel2,
                                produto.codGrupoNivel3, produto.grupoFiscal, produto.suspenderPedidos,
                                produto.idnForaLinha, produto.dataBaixa, produto.refFabrica,
                                produto.idnRevenda, produto.descricaoClasse, produto.linha,
                                produto.coluna, produto.idnGrade, produto.idnFoto,
                                produto.codbarras, produto.preco_minimo, produto.custo,
                                produto.obs, produto.markupMinimo, produto.markupPadrao,
                                produto.markupVenda, produto.sequencia, produto.pesoProduto,
                                produto.pesoEmbalagem, produto.ecommerceAtivo, produto.secao,
                                produto.largura, produto.altura, produto.comprimento,
                                produto.produto_aplicacao, produto.produto_clas_aplicacao
                            ],
                            (error) => {
                                if (error) {
                                    console.error("Error inserting product:", error);
                                    return reject(error);
                                }
                                console.log(`Produto inserido: ${produto.codProduto}`);
                                resolve();
                            }
                        );
                    }
                }
            );
        });
    }
    
        return { success: `Sincronizacao concluida`, code: 200 }

      } catch (error) {
        console.error("ERRO DO CATCH: ", error);
        
        console.error('Erro ao testar SOAP:', error.message);
        if (error.response) console.error('Resposta do servidor:', error.response.data);
      }


    }

    async getPrecos() {
      const soapAction = 'http://www.ultrasistemas.com.br/';
      const soapEnvelope = `<?xml version="1.0" encoding="utf-8"?>
<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">
  <soap:Body>
    <GetPrecosTabela xmlns="http://www.ultrasistemas.com.br/">
      <dataHora>18.12.2024 13:00</dataHora>
    </GetPrecosTabela>
  </soap:Body>
</soap:Envelope>`;
      
      
        try {
          const response = await axios.post(this.wsdlUrl, soapEnvelope, {
            headers: {
              'Content-Type': 'application/soap+xml; charset=utf-8',
              'Content-Length': soapEnvelope.length,
              'SOAPAction': soapAction, // SOAP Action obrigatório
            },
          });
      
          // console.log('SOAP Response: ', response);
          // return response.data
          const parser = new xml2js.Parser({ explicitArray: false });
    const jsonResponse = await parser.parseStringPromise(response.data);

    console.log('SOAP Response (JSON):', jsonResponse);

    return jsonResponse;
        } catch (error) {
          console.error('Erro ao testar SOAP:', error.message);
          if (error.response) {
            console.error('Resposta do servidor:', error.response.data);
          }
        }
      }

  

  async getDetails(args) {
    try {
      const client = await this.createClient();
      const [result] = await client.GetDetailsAsync(args);
      return result;
    } catch (error) {
      console.error('Erro na requisição SOAP:', error);
      throw error;
    }
  }

}

module.exports = usersRepository;
